const MessageDelegate = require('./message');
const Async = require('sketch/async');
var UI = require('sketch/ui')
var sketch = require('sketch')
var Rectangle = require('sketch/dom').Rectangle
var ShapePath = require('sketch/dom').ShapePath
var Style = sketch.Style

var fiber;
var _window;
var contextDocument;

// Main function. Called when Plugin is opened
function onRun(context){
	contextDocument = context.document
	if(!fiber){
		fiber = Async.createFiber();
		fiber.onCleanup(() => {
			closePlugin();
		});
	}
	loadWindow(context.scriptURL);
};

// Get visible page area
var getVisibleArea = function(document) {
	try {
		if (document.currentView) {
			return document.currentView();
		  } 
		  else if (document.contentDrawView) {
			return document.contentDrawView();
		  }
		  return null;
	}
	catch (error) {
		return null
	}
  }
  

// Parsing incoming messages from UI
function parseMessage(message) {
	if (message.type === 'error') {
		UI.message(message.content)
	}
	if (message.type === 'imageContent') {
		if (message.content === undefined || message.width === undefined || message.height === undefined || message.width === 0 || message.height === 0) {
			UI.message("Can't insert the image.")
			return
		}
		try {
			var document = sketch.getSelectedDocument()
			var page = document.selectedPage
			let imageData = NSData.alloc().initWithBase64EncodedString_options(message.content, NSDataBase64DecodingIgnoreUnknownCharacters);
			let nsimage = NSImage.alloc().initWithData(imageData);

			var selectedLayers = document.selectedLayers

			if (selectedLayers.length > 0) {
				selectedLayers.forEach(function (layer, i) {
					layer.style = { 
						fills: [{ 
						fill: 'Pattern',
						pattern: {
							patternType: Style.PatternFillType.Fill,
							image: nsimage,
						}
						}]
					}
				})
			}
			else {
				var view = getVisibleArea(contextDocument);
				var centerX = 0
				var centerY = 0

				if (view !== null){
					var visibleContent = view.visibleContentRect();
					centerX = visibleContent.origin.x;
					centerY = visibleContent.origin.y;
				}
				
				let rectangle = new ShapePath({
				frame: new Rectangle(centerX, centerY, message.width, message.height),
				style: { 
					fills: [{ 
					fill: 'Pattern',
					pattern: {
						patternType: Style.PatternFillType.Fill,
						image: nsimage,
					}
					}]
				},
				parent: page
				})
			}
			
		}
		catch (error) {
			UI.message("Can't insert the image.")
		}
	}
}

// Create Web view to get messages from UI
function createWebView(pageURL, parseMessage, onLoadFinish){
	const webView = WKWebView.alloc().init();

	const delegate = new MessageDelegate({
		"webView:didFinishNavigation:": (webView, navigation) => {
			onLoadFinish();
		},
		"userContentController:didReceiveScriptMessage:": (_, wkMessage) => {
			const message = JSON.parse(wkMessage.body());

			parseMessage(message);
		}
	}).getClassInstance();

	webView.navigationDelegate = delegate;

	const userContentController = webView.configuration().userContentController();

	userContentController.addScriptMessageHandler_name(delegate, "sketchPlugin");

	webView.loadFileURL_allowingReadAccessToURL(pageURL, pageURL.URLByDeletingLastPathComponent());

	return webView;
};


// Create main window
function createWindow(){
	const window = NSPanel.alloc().initWithContentRect_styleMask_backing_defer(
		NSMakeRect(100, 100, 400, 600),
		NSWindowStyleMaskClosable | NSWindowStyleMaskTitled,
		NSBackingStoreBuffered,
		false
	);

	window.becomesKeyOnlyIfNeeded = true;
	window.floatingPanel = true;
	window.releasedWhenClosed = false;

	return window;
};


// Show window when created
function showWindow(window){
	window.makeKeyAndOrderFront(nil);
    
};


// Create plugin window logic or return current
function loadWindow(baseURL){
	if(_window){
		showWindow(_window);
		return;
	}

	const pageURL = baseURL
		.URLByDeletingLastPathComponent()
		.URLByAppendingPathComponent(`./ui/ui.html`);

	const window = createWindow();
	const webView = createWebView(pageURL, parseMessage, () => {
		showWindow(window);
	});

	window.contentView = webView;

	_window = window;
};


// Close Plugin logic
function closePlugin(){
	if(_window){
		_window.orderOut(nil);
		_window = null;
	}
};
